import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;

import javax.swing.*;

/**
 * A simple GUI for selecting the start and end point for 
 * graph traversal algorithm between the 48 US states, and
 * invoking one of the three algorithms: Breadth First Search,
 * Depth First Search, or Shortest Path
 */
public class GraphAlgoView<T extends Node> extends Display implements JPTConstants{

    /** The US map that will be used in the graph traversals */
    Graph<T> graph;
  
    /** a list of all nodes */
    protected String[] labels;

    /** The view to select the start state */
    protected DropdownView fromView;

    /** The view to select the start state */
    protected DropdownView toView;

    /** The display to hold the view for selecting the start state */
    protected ScrollableDisplay fromDisplay = 
        new ScrollableDisplay(fromView);

    /** The display to hold the view for selecting the start state */
    protected ScrollableDisplay toDisplay = 
        new ScrollableDisplay(toView);

    /** A list of three choices of algorithms for user to select */
    protected String[] algoChoices = 
        new String[]{"Breadth First Search", 
                     "Depth First Search", 
                     "Shortest Path"};

    /** perform breadth first search when this choice is made */
    SimpleAction bfsAction = new SimpleAction(){
            public void perform(){
                bfs(fromView.getViewState(), toView.getViewState());
            };
        };

    /** perform depth first search when this choice is made */
    SimpleAction dfsAction = new SimpleAction(){
            public void perform(){
                dfs(fromView.getViewState(), toView.getViewState());
            };
        };

    /** perform shortest path search when this choice is made */
    SimpleAction spAction = new SimpleAction(){
            public void perform(){
                sp(fromView.getViewState(), toView.getViewState());
            };
        };

    /** The list of three actions the program can take */
    protected Action[] algoActions = 
        new Action[]{bfsAction, dfsAction, spAction};

    /** The view for selecting which algorithm to perform */
    protected OptionsView algoChoiceView = 
        new OptionsView(algoChoices, algoActions);
  
  
    /** The class that implements the three algorithms we designed */
    GraphAlgorithms<T> ga;

    /** a stub for the breadth first search algorithm: 
     * replace the body with a call to your algorithm implementation */
    public void bfs(String fromNode, String toNode){
        System.out.println("BFS selected: from " + fromNode + " to " + toNode);
        ga.findRouteBFS(fromNode, toNode);
        System.out.print(this.ga.path.directions(toNode));
    }

    /** a stub for the depth first search algorithm: 
     * replace the body with a call to your algorithm implementation */
    public void dfs(String fromNode, String toNode){
        System.out.println("DFS selected: from " + fromNode + " to " + toNode);
        ga.findRouteDFS(fromNode, toNode);
        System.out.print(this.ga.path.directions(toNode));
    }

    /** a stub for the shortest path search algorithm: 
     * replace the body with a call to your algorithm implementation */
    public void sp(String fromNode, String toNode){
        System.out.println("SPS selected: from " + fromNode + " to " + toNode);
        ga.findRouteSPS(fromNode, toNode);
        System.out.print(this.ga.path.directions(toNode));
    }

    /**
     * The constructor
     */
    public GraphAlgoView(Graph<T> graph){
        super();
        this.graph = graph;
        this.ga = new GraphAlgorithms<T>(this.graph);
        initDropdownViews();

        // add the internal panel to the Display
        add(createDisplay());

        // set the title and annotation for the Display
        setTitleText("Graph Algorithms");
    }
  
    /**
     * Initialize the dropdown views to display the choices of 
     * all node labels.
     */
    protected void initDropdownViews(){
        String[] labels = new String[50];
        labels = (String[])this.graph.data.keySet().toArray(labels);
        this.fromView = new DropdownView(labels);
        this.toView = new DropdownView(labels);
    }

    /**
     * Construct a table panel display for selecting
     * from and to state and the algorithm to run
     */
    protected TablePanel createDisplay(){
        TablePanel graphAlgoDisplay = new TablePanel(new Object[][]{
                {"From:", this.fromView},
                {"To:", this.toView},
                {"Algorithm:", this.algoChoiceView}},
            VERTICAL, 5, 5);

        return graphAlgoDisplay;        	
    }

    /** Run a simple test */
    public static void main(String[]argv){
        Graph<State> us = new USmap();
        GraphAlgoView<State> graphAlgoGUI = new GraphAlgoView<State>(us);
        GeneralDialog.showOneButtonDialog(graphAlgoGUI, "Travel US", "Exit");

        Graph<EuroState> euro = new EuroGraph();
        GraphAlgoView<EuroState> graphAlgoGUI2 = new GraphAlgoView<EuroState>(euro);
        GeneralDialog.showOneButtonDialog(graphAlgoGUI2, "Travel Euro", "Exit");
    }

}